@@ -130,6 +130,8 @@ typedef struct {
     const char **result_string;
     /** Arbitrary context data provided by the caller for custom functions */
     void *data;
+    /** The current recursion level */
+    int reclvl;
 } ap_expr_eval_ctx_t;
 
 /**
