@@ -28,14 +28,13 @@
 #include "apr_thread_rwlock.h"
 #include "apr_tables.h"
 #include "apr_time.h"
+#include "apr_version.h"
+#if APR_MAJOR_VERSION < 2
+/* The LDAP API is currently only present in APR 1.x */
 #include "apr_ldap.h"
 #include "apr_ldap_rebind.h"
-
-#if APR_HAS_MICROSOFT_LDAPSDK
-#define AP_LDAP_IS_SERVER_DOWN(s)                ((s) == LDAP_SERVER_DOWN \
-                ||(s) == LDAP_UNAVAILABLE)
 #else
-#define AP_LDAP_IS_SERVER_DOWN(s)                ((s) == LDAP_SERVER_DOWN)
+#define APR_HAS_LDAP 0
 #endif
 
 #if APR_HAS_SHARED_MEMORY
@@ -46,6 +45,13 @@
 /* this whole thing disappears if LDAP is not enabled */
 #if APR_HAS_LDAP
 
+#if APR_HAS_MICROSOFT_LDAPSDK
+#define AP_LDAP_IS_SERVER_DOWN(s)                ((s) == LDAP_SERVER_DOWN \
+                ||(s) == LDAP_UNAVAILABLE)
+#else
+#define AP_LDAP_IS_SERVER_DOWN(s)                ((s) == LDAP_SERVER_DOWN)
+#endif
+
 /* Apache header files */
 #include "ap_config.h"
 #include "httpd.h"
@@ -56,7 +62,7 @@
 #include "http_request.h"
 #include "apr_optional.h"
 
-/* Create a set of LDAP_DECLARE macros with appropriate export 
+/* Create a set of LDAP_DECLARE macros with appropriate export
  * and import tags for the platform
  */
 #if !defined(WIN32)
@@ -87,9 +93,9 @@ extern "C" {
 
 /* Values that the deref member can have */
 typedef enum {
-    never=LDAP_DEREF_NEVER, 
-    searching=LDAP_DEREF_SEARCHING, 
-    finding=LDAP_DEREF_FINDING, 
+    never=LDAP_DEREF_NEVER,
+    searching=LDAP_DEREF_SEARCHING,
+    finding=LDAP_DEREF_FINDING,
     always=LDAP_DEREF_ALWAYS
 } deref_options;
 
@@ -121,6 +127,8 @@ typedef struct util_ldap_connection_t {
 
     int ChaseReferrals;                 /* [on|off] (default = AP_LDAP_CHASEREFERRALS_ON)*/
     int ReferralHopLimit;               /* # of referral hops to follow (default = AP_LDAP_DEFAULT_HOPLIMIT) */
+    apr_time_t freed;                   /* the time this conn was placed back in the pool */
+    apr_pool_t *rebind_pool;            /* frequently cleared pool for rebind data */
 } util_ldap_connection_t;
 
 typedef struct util_ldap_config_t {
@@ -129,7 +137,7 @@ typedef struct util_ldap_config_t {
     apr_array_header_t *client_certs;  /* Client certificates */
 } util_ldap_config_t;
 
-/* LDAP cache state information */ 
+/* LDAP cache state information */
 typedef struct util_ldap_state_t {
     apr_pool_t *pool;           /* pool from which this state is allocated */
 #if APR_HAS_THREADS
@@ -163,7 +171,9 @@ typedef struct util_ldap_state_t {
     struct timeval *opTimeout;
 
     int debug_level;                    /* SDK debug level */
-
+    apr_interval_time_t connection_pool_ttl;
+    int retries;                        /* number of retries for failed bind/search/compare */
+    apr_interval_time_t retry_delay;    /* delay between retries of failed bind/search/compare */
 } util_ldap_state_t;
 
 /* Used to store arrays of attribute labels/values. */
@@ -183,7 +193,7 @@ struct mod_auth_ldap_groupattr_entry_t {
  * @fn int util_ldap_connection_open(request_rec *r,
  *                                        util_ldap_connection_t *ldc)
  */
-APR_DECLARE_OPTIONAL_FN(int,uldap_connection_open,(request_rec *r, 
+APR_DECLARE_OPTIONAL_FN(int,uldap_connection_open,(request_rec *r,
                                             util_ldap_connection_t *ldc));
 
 /**
@@ -208,15 +218,6 @@ APR_DECLARE_OPTIONAL_FN(void,uldap_connection_close,(util_ldap_connection_t *ldc
  */
 APR_DECLARE_OPTIONAL_FN(apr_status_t,uldap_connection_unbind,(void *param));
 
-/**
- * Cleanup a connection to an LDAP server
- * @param ldc A structure containing the expanded details of the server
- *            that was connected.
- * @tip This functions unbinds and closes the connection to the LDAP server
- * @fn apr_status_t util_ldap_connection_cleanup(util_ldap_connection_t *ldc)
- */
-APR_DECLARE_OPTIONAL_FN(apr_status_t,uldap_connection_cleanup,(void *param));
-
 /**
  * Find a connection in a list of connections
  * @param r The request record
@@ -225,7 +226,7 @@ APR_DECLARE_OPTIONAL_FN(apr_status_t,uldap_connection_cleanup,(void *param));
  * @param binddn The DN to bind with
  * @param bindpw The password to bind with
  * @param deref The dereferencing behavior
- * @param secure use SSL on the connection 
+ * @param secure use SSL on the connection
  * @tip Once a connection is found and returned, a lock will be acquired to
  *      lock that particular connection, so that another thread does not try and
  *      use this connection while it is busy. Once you are finished with a connection,
@@ -256,8 +257,8 @@ APR_DECLARE_OPTIONAL_FN(util_ldap_connection_t *,uldap_connection_find,(request_
  *                                        const char *url, const char *dn, const char *reqdn,
  *                                        int compare_dn_on_server)
  */
-APR_DECLARE_OPTIONAL_FN(int,uldap_cache_comparedn,(request_rec *r, util_ldap_connection_t *ldc, 
-                              const char *url, const char *dn, const char *reqdn, 
+APR_DECLARE_OPTIONAL_FN(int,uldap_cache_comparedn,(request_rec *r, util_ldap_connection_t *ldc,
+                              const char *url, const char *dn, const char *reqdn,
                               int compare_dn_on_server));
 
 /**
@@ -267,7 +268,7 @@ APR_DECLARE_OPTIONAL_FN(int,uldap_cache_comparedn,(request_rec *r, util_ldap_con
  * @param url The URL of the LDAP connection - used for deciding which cache to use.
  * @param dn The DN of the object in which we do the compare.
  * @param attrib The attribute within the object we are comparing for.
- * @param value The value of the attribute we are trying to compare for. 
+ * @param value The value of the attribute we are trying to compare for.
  * @tip Use this function to determine whether an attribute/value pair exists within an
  *      object. Typically this would be used to determine LDAP top-level group
  *      membership.
