@@ -213,7 +213,7 @@ static int find_code_page(request_rec *r)
     /* If we don't have a full directory configuration, bail out.
      */
     if (!dc->charset_source || !dc->charset_default) {
-        ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
+        ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, APLOGNO(01448)
                       "incomplete configuration: src %s, dst %s",
                       dc->charset_source ? dc->charset_source : "unspecified",
                       dc->charset_default ? dc->charset_default : "unspecified");
@@ -249,7 +249,7 @@ static int find_code_page(request_rec *r)
 
     reqinfo->dc = dc;
     output_ctx->dc = dc;
-    output_ctx->tmpbb = apr_brigade_create(r->pool, 
+    output_ctx->tmpbb = apr_brigade_create(r->pool,
                                            r->connection->bucket_alloc);
     ap_set_module_config(r->request_config, &charset_lite_module, reqinfo);
 
@@ -271,7 +271,7 @@ static int find_code_page(request_rec *r)
         rv = apr_xlate_open(&input_ctx->xlate, dc->charset_source,
                             dc->charset_default, r->pool);
         if (rv != APR_SUCCESS) {
-            ap_log_rerror(APLOG_MARK, APLOG_ERR, rv, r,
+            ap_log_rerror(APLOG_MARK, APLOG_ERR, rv, r, APLOGNO(01449)
                           "can't open translation %s->%s",
                           dc->charset_default, dc->charset_source);
             return HTTP_INTERNAL_SERVER_ERROR;
@@ -319,7 +319,7 @@ static void xlate_insert_filter(request_rec *r)
     charset_dir_t *dc = ap_get_module_config(r->per_dir_config,
                                              &charset_lite_module);
 
-    if (dc && (dc->implicit_add == IA_NOIMPADD)) { 
+    if (dc && (dc->implicit_add == IA_NOIMPADD)) {
         ap_log_rerror(APLOG_MARK, APLOG_TRACE6, 0, r,
                       "xlate output filter not added implicitly because "
                       "CharsetOptions included 'NoImplicitAdd'");
@@ -479,19 +479,19 @@ static void log_xlate_error(ap_filter_t *f, apr_status_t rv)
     switch(ctx->ees) {
     case EES_LIMIT:
         rv = 0;
-        msg = "xlate filter - a built-in restriction was encountered";
+        msg = APLOGNO(02193) "xlate filter - a built-in restriction was encountered";
         break;
     case EES_BAD_INPUT:
         rv = 0;
-        msg = "xlate filter - an input character was invalid";
+        msg = APLOGNO(02194) "xlate filter - an input character was invalid";
         break;
     case EES_BUCKET_READ:
         rv = 0;
-        msg = "xlate filter - bucket read routine failed";
+        msg = APLOGNO(02195) "xlate filter - bucket read routine failed";
         break;
     case EES_INCOMPLETE_CHAR:
         rv = 0;
-        strcpy(msgbuf, "xlate filter - incomplete char at end of input - ");
+        strcpy(msgbuf, APLOGNO(02196) "xlate filter - incomplete char at end of input - ");
         cur = 0;
         while ((apr_size_t)cur < ctx->saved) {
             apr_snprintf(msgbuf + strlen(msgbuf), sizeof(msgbuf) - strlen(msgbuf),
@@ -501,13 +501,12 @@ static void log_xlate_error(ap_filter_t *f, apr_status_t rv)
         msg = msgbuf;
         break;
     case EES_DOWNSTREAM:
-        msg = "xlate filter - an error occurred in a lower filter";
+        msg = APLOGNO(02197) "xlate filter - an error occurred in a lower filter";
         break;
     default:
-        msg = "xlate filter - returning error";
+        msg = APLOGNO(02198) "xlate filter - returning error";
     }
-    ap_log_rerror(APLOG_MARK, APLOG_ERR, rv, f->r,
-                  "%s", msg);
+    ap_log_rerror(APLOG_MARK, APLOG_ERR, rv, f->r, "%s", msg);
 }
 
 /* chk_filter_chain() is called once per filter instance; it tries to
@@ -582,7 +581,7 @@ static void chk_filter_chain(ap_filter_t *f)
                             const char *symbol = output ? "->" : "<-";
 
                             ap_log_rerror(APLOG_MARK, APLOG_DEBUG,
-                                          0, f->r,
+                                          0, f->r, APLOGNO(01451)
                                           "%s %s - disabling "
                                           "translation %s%s%s; existing "
                                           "translation %s%s%s",
@@ -600,7 +599,7 @@ static void chk_filter_chain(ap_filter_t *f)
                         const char *symbol = output ? "->" : "<-";
 
                         ap_log_rerror(APLOG_MARK, APLOG_ERR,
-                                      0, f->r,
+                                      0, f->r, APLOGNO(01452)
                                       "chk_filter_chain() - can't disable "
                                       "translation %s%s%s; existing "
                                       "translation %s%s%s",
@@ -805,13 +804,13 @@ static apr_status_t xlate_out_filter(ap_filter_t *f, apr_bucket_brigade *bb)
          */
             strcmp(mime_type, DIR_MAGIC_TYPE) == 0 ||
 #endif
-            strncasecmp(mime_type, "message/", 8) == 0 || 
+            strncasecmp(mime_type, "message/", 8) == 0 ||
             dc->force_xlate == FX_FORCE)) {
 
             rv = apr_xlate_open(&ctx->xlate,
                                 dc->charset_default, dc->charset_source, f->r->pool);
             if (rv != APR_SUCCESS) {
-                ap_log_rerror(APLOG_MARK, APLOG_ERR, rv, f->r,
+                ap_log_rerror(APLOG_MARK, APLOG_ERR, rv, f->r, APLOGNO(01453)
                               "can't open translation %s->%s",
                               dc->charset_source, dc->charset_default);
                 ctx->noop = 1;
